@@ -28,6 +28,7 @@ void CommandData::Init()
 
   ListMode=RCLM_AUTO;
 
+
   FileArgs=new StringList;
   ExclArgs=new StringList;
   InclArgs=new StringList;
@@ -252,7 +253,9 @@ void CommandData::ParseEnvVar()
 {
   char *EnvStr=getenv("RAR");
   if (EnvStr!=NULL)
+  {
     ProcessSwitchesString(EnvStr);
+  }
 }
 #endif
 
@@ -651,13 +654,16 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
     case 'P':
       if (Switch[1]==0)
       {
-        GetPassword(PASSWORD_GLOBAL,NULL,NULL,Password,ASIZE(Password));
+        GetPassword(PASSWORD_GLOBAL,NULL,NULL,&Password);
         eprintf("\n");
       }
       else
       {
-        CharToWide(Switch+1,Password,ASIZE(Password));
-        Password[ASIZE(Password)-1]=0;
+        wchar PlainPsw[MAXPASSWORD];
+        CharToWide(Switch+1,PlainPsw,ASIZE(PlainPsw));
+        PlainPsw[ASIZE(PlainPsw)-1]=0;
+        Password.Set(PlainPsw);
+        cleandata(PlainPsw,ASIZE(PlainPsw));
       }
       break;
     case 'H':
@@ -666,13 +672,16 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
         EncryptHeaders=true;
         if (Switch[2]!=0)
         {
-          CharToWide(Switch+2,Password,ASIZE(Password));
-          Password[ASIZE(Password)-1]=0;
+          wchar PlainPsw[MAXPASSWORD];
+          CharToWide(Switch+2,PlainPsw,ASIZE(PlainPsw));
+          PlainPsw[ASIZE(PlainPsw)-1]=0;
+          Password.Set(PlainPsw);
+          cleandata(PlainPsw,ASIZE(PlainPsw));
         }
         else
-          if (*Password==0)
+          if (!Password.IsSet())
           {
-            GetPassword(PASSWORD_GLOBAL,NULL,NULL,Password,ASIZE(Password));
+            GetPassword(PASSWORD_GLOBAL,NULL,NULL,&Password);
             eprintf("\n");
           }
       }
